/*
  main.c - Simple multithreaded demo application

  Part of FreeRTOS NIOS2 port - Copyright (C) 2006 Kolja Waschk

  This port is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This port is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  A special exception to the GPL can be applied should you wish to distribute
  a combined work that includes this port, without being obliged to provide
  the source code for any proprietary components.  See the licensing section 
  of http://www.FreeRTOS.org for full details of how and when the exception
  can be applied.

  You should have received a copy of the GNU General Public License
  along with FreeRTOS.org; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>

#include "FreeRTOS.h"
#include "task.h"

#include <reent.h>
extern void vPortInitReent(struct _reent *pREENT);

int signal_to_other_task = 0;

/* ====================================== */

static portTASK_FUNCTION (vSign, pvParameters)
{
  /* The parameters are not used. */
  (void)pvParameters;

  for(;;)
  {
    volatile unsigned long i;
#if ( configUSE_PREEMPTION == 1 )
    for(i=0;i<10000000L;i++);
    signal_to_other_task++;
#else
    for(i=0;i<500000L;i++) vTaskDelay(0); /* Yield */
    signal_to_other_task++;
#endif    
  };
}

/* ====================================== */

static portTASK_FUNCTION (vMoin, pvParameters)
{
  struct _reent prvREENT;

  /* The parameters are not used. */
  (void)pvParameters;
  
  vPortInitReent(&prvREENT);

  /* Be polite. */
  for(;;)
  {
    printf("Moin, Porter (%d).\n", signal_to_other_task);
    signal_to_other_task = 0;
    vTaskDelay( 1 * configTICK_RATE_HZ ); /* delay 1 second */
  };
}

/* ====================================== */

static portTASK_FUNCTION (vMain, pvParameters)
{
  struct _reent prvREENT;

  /* The parameters are not used. */
  (void)pvParameters;

  vPortInitReent(&prvREENT);

  printf("%s\n", (char *)pvParameters);
 
  /* Be polite. */
  for(;;)
  {
    printf("Look back.\n");
    vTaskDelay( 2 * configTICK_RATE_HZ ); /* delay 2 second */
  };
}

int main(int argc, char **argv, char **envp)
{
  int xReturn;
  
  /* Initialise the operating system,
   * create the 'Main' task to be run beside 'Idle'. */

  xReturn = xTaskCreate( vMain, "Main", configMINIMAL_STACK_SIZE + sizeof(struct _reent), "Main arg.", tskIDLE_PRIORITY + 2, NULL );
  xReturn = xTaskCreate( vMoin, "Moin", configMINIMAL_STACK_SIZE + sizeof(struct _reent), NULL, tskIDLE_PRIORITY + 2, NULL );
  xReturn = xTaskCreate( vSign, "Sign", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 1, NULL );

  /* Start the Scheduler. It will create the "Idle" task and usually doesn't return */
  vTaskStartScheduler();
  
  return 123; /* not reached */
}
